--
-- © Copyright Peregrine I.T. Pty Ltd 2004-2018 except where explicitly stated otherwise.
-- Fantasy Grounds is Copyright © 2004-2018 SmiteWorks USA LLC.
-- Copyright to other material within this file may be held by other Individuals and/or Entities.
-- Nothing in or from this LUA file in printed, electronic and/or any other form may be used, copied,
--	transmitted or otherwise manipulated in ANY way without the explicit written consent of Matthew
--	Peregrine I.T. Pty Ltd or, where applicable, any and all other Copyright holders.
-- Please see the accompanying License for full details.
-- All rights reserved.
--
-- Called By: xREMUFiles.xml
--
DBVERSION = 1;
EXTENSION = "REMU";

aDate = {};
sBtnREMU = "btnREMUCoreRPG";
sBtnREMUDown = "btnREMUDownCoreRPG";
sBtnREMUUpdate = "btnREMUUpdateCoreRPG";
sErrorLog = "";
sLog = "";

local aFileLocations = {
	["sManifests"] = nil,
	["sRulesets"] = nil,
	["sExtensions"] = nil,
	["sModules"] = nil
	};
local aManifest = {};
local nImportCount = 0;
local sManifestFileName = nil;
local sManifestFileNameTmp = nil;
local sManifestVersionTmp = nil;
local sManifestURLTmp = nil;

function onInit()
	local oNode = DB.findNode("MJB_Extensions");
	if User.isHost() then
		if oNode == nil or
				DB.getChild(oNode,EXTENSION) == nil then
			DB.setValue(DB.createNode("MJB_Extensions"),EXTENSION,"number",DBVERSION);
			DB.createNode("MJB_REMU");
		elseif DB.getValue(oNode,EXTENSION) < DBVERSION then
			fpDoPatch2();
		end
	end
	fpSetRulesetGraphics();
	fpMain();
	return;
end

function fpSetRulesetGraphics()
	local sRuleset = Interface.getRuleset();
	sRuleset = "CoreRPG";
	sBtnREMU = "btnREMU" .. sRuleset;
	sBtnREMUDown = "btnREMUDown" .. sRuleset;
	sBtnREMUUpdate = "btnREMUUpdate" .. sRuleset;
end

-- Patch Functions --

function fpDoPatch2()
	DB.backup();
	return;
end

-- End Of Patch Functions --

function fpAbortRegistration()
	fpWriteLog("error","Manifest File: " .. sManifestFileNameTmp .. " already exists. Manifest not Registered.");
	return;
end

function fpContinueDeregistration()
	local oWindow = Interface.findWindow("wcManifestWindow","");
	if oWindow then
		for kKey,vValue in pairs(oWindow.swManifestsLists.subwindow.wlList.getWindows()) do
			if vValue.nDelete.getValue() == 1 then
				sManifestFileNameTmp = vValue.name.getValue();
				fpDeregCleanUp("aRulesets");
				fpDeregCleanUp("aExtensions");
				fpDeregCleanUp("aModules");
				GlobalRegistry.MJB_REMU["aManifests"][sManifestFileNameTmp] = nil;
				vValue.close();
			end
			sManifestFileNameTmp = nil;
		end
	end
	local oWindow = Interface.findWindow("wcREMUWindow","");
	if oWindow then
		oWindow.swREMULists.subwindow.fpUpdate();
	end
	return;
end

function fpContinueRegistration()
	local aManifests = GlobalRegistry.MJB_REMU["aManifests"];
	local oNode = DB.findNode("MJB_REMU");
	aManifests[sManifestFileNameTmp] = GlobalRegistry.MJB_REMU["aManifests"][sManifestFileNameTmp] or {};
	aManifests[sManifestFileNameTmp]["sURL"] = oNode.getChild("sURL").getValue();
	aManifests[sManifestFileNameTmp]["sProtocol"] = oNode.getChild("sProtocol").getValue();
	aManifests[sManifestFileNameTmp]["sVersion"] = oNode.getChild("sVersion").getValue();
	fpUpdateManifest(sManifestFileNameTmp);
	fpWriteLog("success","Manifest File: Manifest " .. sManifestFileNameTmp .. " succesfully Registered.");
	return;
end

function fpCreateHelpers()
	aFileLocations["sManifests"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/manifests/";
	aFileLocations["sRulesets"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/rulesets/";
	aFileLocations["sExtensions"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/extensions/";
	aFileLocations["sModules"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/modules/";
	local sHelperString1 = "C:\\DOEREMUHelpers\\paste.exe > C:\\DOEREMUHelpers\\";
	local sHelperString2 = "_URLs.txt\nC:\\DOEREMUHelpers\\wget.exe -i C:\\DOEREMUHelpers\\";
	local sHelperString3 = "_URLs.txt -o C:\\DOEREMUHelpers\\LogFile.txt --no-check-certificate -N -P \"";
	local sHelperVar = "Manifest";
	Interface.clipboardSetText(sHelperString1 .. sHelperVar .. sHelperString2 .. sHelperVar .. sHelperString3 .. aFileLocations["sManifests"]);
	Interface.openWindow("url","C:/DOEREMUHelpers/helper01.bat");
	MiscFunctions.fpSleep(5);
	sHelperVar = "Ruleset";
	Interface.clipboardSetText(sHelperString1 .. sHelperVar .. sHelperString2 .. sHelperVar .. sHelperString3 .. aFileLocations["sRulesets"]);
	Interface.openWindow("url","C:/DOEREMUHelpers/helper02.bat");
	MiscFunctions.fpSleep(5);
	sHelperVar = "Extension";
	Interface.clipboardSetText(sHelperString1 .. sHelperVar .. sHelperString2 .. sHelperVar .. sHelperString3 .. aFileLocations["sExtensions"]);
	Interface.openWindow("url","C:/DOEREMUHelpers/helper03.bat");
	MiscFunctions.fpSleep(5);
	sHelperVar = "Module";
	Interface.clipboardSetText(sHelperString1 .. sHelperVar .. sHelperString2 .. sHelperVar .. sHelperString3 .. aFileLocations["sModules"]);
	Interface.openWindow("url","C:/DOEREMUHelpers/helper04.bat");
	return;
end

function fpCreateManifest(oWindow)
	if not oWindow then
		return;
	end
	local oNode = DB.findNode("MJB_REMU");
	if oNode then
		DB.deleteNode(oNode);
	end
	oNode = DB.createNode("MJB_REMU");
	local sManifestName = oWindow.sManifestFileName.getValue();
	if oWindow.nNew.getValue() == 1 then
		sManifestName = oWindow.sNewManifestFileName.getValue();
	end
	oNode.createChild("sFileName","string").setValue(sManifestName);
	oNode.createChild("sURL","string").setValue(fpParseURLString(oWindow.sURL.getValue()));
	oNode.createChild("sProtocol","string").setValue(oWindow.sProtocol.getValue());
	oNode.createChild("sVersion","string").setValue(oWindow.sVersion.getValue());
	if oWindow.swManifestREMULists.subwindow.swRulesets.subwindow.wlList.getWindowCount(true) > 0 then
		fpCreateManifestDetails(oNode.createChild("rulesets"),oWindow.swManifestREMULists.subwindow.swRulesets.subwindow.wlList);
	end
	if oWindow.swManifestREMULists.subwindow.swExtensions.subwindow.wlList.getWindowCount(true) > 0 then
		fpCreateManifestDetails(oNode.createChild("extensions"),oWindow.swManifestREMULists.subwindow.swExtensions.subwindow.wlList);
	end
	if oWindow.swManifestREMULists.subwindow.swModules.subwindow.wlList.getWindowCount(true) > 0 then
		fpCreateManifestDetails(oNode.createChild("modules"),oWindow.swManifestREMULists.subwindow.swModules.subwindow.wlList);
	end
	sManifestFileNameTmp = sManifestName;
	fpContinueRegistration();
	DB.export(aFileLocations["sManifests"] .. sManifestName .. ".rem",oNode);
	fpWriteLog("success","Manifest File: Manifest " .. sManifestName .. " succesfully Created.");
	return;
end

function fpCreateManifestDetails(oNode,wlList)
	for kKey,vValue in pairs(wlList.getWindows(true)) do
		local oChildNode = oNode.createChild();
		oChildNode.createChild("sFileName","string").setValue(vValue.name.getValue());
		oChildNode.createChild("sURL","string").setValue(fpParseURLString(vValue.sURL.getValue()));
		oChildNode.createChild("sProtocol","string").setValue(vValue.sProtocol.getValue());
		oChildNode.createChild("nVersionMajor","number").setValue(vValue.nMajorVer.getValue());
		oChildNode.createChild("nVersionMinor","number").setValue(vValue.nMinorVer.getValue());
		oChildNode.createChild("nVersionPatch","number").setValue(vValue.nPatch.getValue());
		oChildNode.createChild("nDelete","number").setValue(vValue.nRemove.getValue());
	end
	return;
end

function fpDeregCleanUp(sType)
	local oREMWindow = Interface.findWindow("wcREMUWindow","");
	for kKey,vValue in pairs(GlobalRegistry.MJB_REMU[sType]) do
		if vValue["sManifestName"] == sManifestFileNameTmp then
			if oREMWindow then
				if sType == "aRulesets" then
					fpDeregCleanUpRulesets(oREMWindow,kKey);
				elseif sType =="aExtensions" then
					fpDeregCleanUpExtensions(oREMWindow,kKey);
				else
					fpDeregCleanUpModules(oREMWindow,kKey);
				end
			end
			GlobalRegistry.MJB_REMU[sType][kKey] = nil;
		end
	end
	return;
end

function fpDeregCleanUpExtensions(oREMWindow,sName)
	for kKey,vValue in pairs(oREMWindow.swREMULists.subwindow.swExtensions.subwindow.wlList.getWindows()) do
		if vValue.name.getValue() == sName then
			vValue.close();
		end
	end
	return;
end

function fpDeregCleanUpModules(oREMWindow,sName)
	for kKey,vValue in pairs(oREMWindow.swREMULists.subwindow.swModules.subwindow.wlList.getWindows()) do
		if vValue.name.getValue() == sName then
			vValue.close();
		end
	end
	return;
end

function fpDeregCleanUpRulesets(oREMWindow,sName)
	for kKey,vValue in pairs(oREMWindow.swREMULists.subwindow.swRulesets.subwindow.wlList.getWindows()) do
		if vValue.name.getValue() == sName then
			vValue.close();
		end
	end
	return;
end

function fpDeregisterManifest()
	MiscFunctions.fpMsgbox("You are about to Deregister the selected REM Manifests. Are you sure?","Confirm Manifest Deregistration",4,4,fpContinueDeregistration);
	return;
end

function fpImportManifests()
	local oNode = DB.findNode("MJB_REMU");
	if oNode then
		for kKey,vValue in pairs(GlobalRegistry.MJB_REMU["aManifests"]) do
			nImportCount = 0;
			sManifestFileName = kKey;
			aManifest = vValue;
			DB.import(aFileLocations["sManifests"] .. kKey .. ".rem",oNode);
		end
	end
	return;
end

function fpIsUpdate()
	for kKey,vValue in pairs(GlobalRegistry.MJB_REMU["aRulesets"]) do
		if vValue["nIgnore"] == 0 and
				vValue["nUpdateable"] == 1 then
			return true;
		end
	end
	for kKey,vValue in pairs(GlobalRegistry.MJB_REMU["aExtensions"]) do
		if vValue["nIgnore"] == 0 and
				vValue["nUpdateable"] == 1 then
			return true;
		end
	end
	for kKey,vValue in pairs(GlobalRegistry.MJB_REMU["aModules"]) do
		if vValue["nIgnore"] == 0 and
				vValue["nUpdateable"] == 1 then
			return true;
		end
	end
	return false;
end

function fpMain()
	DB.onImport = fpOnImport;
	aDate = os.date("!*t");
	Comm.registerSlashHandler("manifest",fpProcessSlashManifest);
	Comm.registerSlashHandler("remusetup",fpProcessREMUSetup);
	GlobalRegistry.MJB_REMU = GlobalRegistry.MJB_REMU or {};
	GlobalRegistry.MJB_REMU["aManifests"] = GlobalRegistry.MJB_REMU["aManifests"] or {};
	GlobalRegistry.MJB_REMU["aRulesets"] = GlobalRegistry.MJB_REMU["aRulesets"] or {};
	GlobalRegistry.MJB_REMU["aExtensions"] = GlobalRegistry.MJB_REMU["aExtensions"] or {};
	GlobalRegistry.MJB_REMU["aModules"] = GlobalRegistry.MJB_REMU["aModules"] or {};
	GlobalRegistry.MJB_REMU["nSetupDone"] = GlobalRegistry.MJB_REMU["nSetupDone"] or 0;
	GlobalRegistry.MJB_REMU["nUpdateInterval"] = GlobalRegistry.MJB_REMU["nUpdateInterval"] or 7;
	GlobalRegistry.MJB_REMU["nYDayLastCheck"] = GlobalRegistry.MJB_REMU["nYDayLastCheck"] or aDate["yday"];
	GlobalRegistry.MJB_REMU["nYearLastCheck"] = GlobalRegistry.MJB_REMU["nYearLastCheck"] or aDate["year"];
	GlobalRegistry.MJB_REMU["sFGDataDir"] = GlobalRegistry.MJB_REMU["sFGDataDir"] or "%appdata%/fantasy grounds";
	aFileLocations["sManifests"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/manifests/";
	aFileLocations["sRulesets"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/rulesets/";
	aFileLocations["sExtensions"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/extensions/";
	aFileLocations["sModules"] = GlobalRegistry.MJB_REMU["sFGDataDir"] .. "/modules/";
	if GlobalRegistry.MJB_REMU["nYearLastCheck"] < aDate["year"] or
			GlobalRegistry.MJB_REMU["nYDayLastCheck"] + GlobalRegistry.MJB_REMU["nUpdateInterval"] <= aDate["yday"] then
		fpUpdateCheck();
	end
	fpImportManifests()
	if User.isHost() then
		if fpIsUpdate() then
			DesktopManager.registerStackShortcut2(sBtnREMUUpdate,sBtnREMUDown,"sSidebarTooltipREMUStr","wcREMUStartWindow",nil);
		else
			DesktopManager.registerStackShortcut2(sBtnREMU,sBtnREMUDown,"sSidebarTooltipREMUStr","wcREMUStartWindow",nil);
		end
	end
	return;
end

function fpOnImport(oNode)
	nImportCount = nImportCount+1;
	if oNode.getNodeName() == "MJB_REMU.sFileName" then
		sManifestFileNameTmp = oNode.getValue();
	end
	if oNode.getNodeName() == "MJB_REMU.sURL" then
		sManifestURLTmp = oNode.getValue();
	end
	if oNode.getNodeName() == "MJB_REMU.sProtocol" then
		sManifestProtocolTmp = oNode.getValue();
	end
	if oNode.getNodeName() == "MJB_REMU.sVersion" then
		sManifestVersionTmp = oNode.getValue();
	end
	if nImportCount == 7 then
		if sManifestFileName == nil then
			fpWriteLog("success","Imported " .. sManifestFileNameTmp .. ".");
		elseif sManifestFileNameTmp == sManifestFileName then
			if sManifestVersionTmp > aManifest["sVersion"] or
					sManifestVersionTmp == "-1" and
					aManifest["sVersion"] ~= "-1" then
				GlobalRegistry.MJB_REMU["aManifests"][sManifestFileName]["sVersion"] = sManifestVersionTmp;
				if sManifestURLTmp ~= aManifest["sURL"] then
					GlobalRegistry.MJB_REMU["aManifests"][sManifestFileName]["sURL"] = sManifestURLTmp;
				end
				if sManifestProtocolTmp ~= aManifest["sProtocol"] then
					GlobalRegistry.MJB_REMU["aManifests"][sManifestFileName]["sProtocol"] = sManifestProtocolTmp;
				end
				fpUpdateManifest(sManifestFileName);
			end
			fpWriteLog("success","Manifest File: " .. sManifestFileName .. " checked for updates.");
		else
			fpWriteLog("error","Manifest File: Existing Manifest " .. sManifestFileName .. " has downloaded a new version of itself but the new name (" .. sManifestFileNameTmp .. ") is different. Download the new Manifest (" .. sManifestFileNameTmp .. ") manually and Register it and then remove the exiting Manifest (" .. sManifestFileName .. ").");
		end
	end
	return;
end

function fpParseURLString(sURL)
	local sLocalURL = sURL;
	local sLowerURL = string.lower(sURL);
	if string.sub(sLowerURL,1,7) == "http://" then
		sLocalURL = string.sub(sLocalURL,8);
	elseif string.sub(sLowerURL,1,8) == "https://" then
		sLocalURL = string.sub(sLocalURL,9);
	end
	return sLocalURL;
end

function fpProcessSlashManifest(sCommand,sParams)
	Interface.openWindow("wcExportManifest","");
	return;
end

function fpProcessREMUSetup(sCommand,sParams)
	Interface.openWindow("wcREMUSetup","");
	return;
end

function fpRegisterManifest()
	local oNode = DB.findNode("MJB_REMU");
	if oNode then
		DB.deleteNode(oNode);
	end
	oNode = DB.createNode("MJB_REMU");
	DB.import(Interface.dialogFileOpen(false,{["rem"]=".rem"},aFileLocations["sManifests"]),oNode);
	sManifestFileNameTmp = oNode.getChild("sFileName").getValue();
	if sManifestFileNameTmp then
		if GlobalRegistry.MJB_REMU["aManifests"][sManifestFileNameTmp] then
			MiscFunctions.fpMsgbox(sManifestFileNameTmp .. " is already Registered. Would you like to replace the existing Registered Manifest information with the new Manifest?","Manifest Already Registered",4,4,"fpContinueRegistration","fpAbortRegistration");
		else
			fpContinueRegistration();
		end
	else
		fpWriteLog("error","Manifest File: Manifest not Registered.");
	end
	local oWindow = Interface.findWindow("wcREMUWindow","");
	if oWindow then
		oWindow.swREMULists.subwindow.fpUpdate();
	end
	return;
end

function fpSetMenubarButton()
	local oMenuButton = Interface.findWindow("shortcuts","").shortcutbar.subwindow[Interface.getString("sSidebarTooltipREMUStr")];
	oMenuButton.setIcons(sBtnREMU,sBtnREMUDown);
	if fpIsUpdate() then
		oMenuButton.setIcons(sBtnREMUUpdate,sBtnREMUDown);
	end
	return;
end

function fpUpdate()
	fpUpdateFiles("Rulesets");
	fpUpdateFiles("Extensions");
	fpUpdateFiles("Modules");
	local oWindow = Interface.findWindow("wcREMUWindow","");
	if oWindow then
		oWindow.swREMULists.subwindow.swRulesets.subwindow.wlList.fpUpdate("Rulesets");
		oWindow.swREMULists.subwindow.swExtensions.subwindow.wlList.fpUpdate("Extensions");
		oWindow.swREMULists.subwindow.swModules.subwindow.wlList.fpUpdate("Modules");
	end
	return;
end

function fpUpdateCheck()
	GlobalRegistry.MJB_REMU["nYearLastCheck"] = aDate["year"];
	GlobalRegistry.MJB_REMU["nYDayLastCheck"] = aDate["yday"];
	local sURLStrings = "";
	for kKey,vValue in pairs(GlobalRegistry.MJB_REMU["aManifests"]) do
		sURLStrings = sURLStrings .. vValue["sProtocol"] .. "://" .. vValue["sURL"] .. "\n";
	end
	if sURLStrings then
		Interface.clipboardSetText(sURLStrings);
		Interface.openWindow("url","C:/DOEREMUHelpers/helper07.bat");
		MiscFunctions.fpSleep(5);
		fpImportManifests()
	end
	fpWriteLog("success","Finished checking for updates.");
	return;
end

function fpUpdateFiles(sType)
	local sURLStrings = "";
	for kKey,vValue in pairs(GlobalRegistry.MJB_REMU["a".. sType]) do
		if vValue["nIgnore"] == 0 and
				vValue["nUpdateable"] == 1 then
			sURLStrings = sURLStrings .. vValue["sProtocol"] .. "://" .. vValue["sURL"] .. "\n";
			vValue["nVersionMajor"] = vValue["nVersionMajorNew"];
			vValue["nVersionMinor"] = vValue["nVersionMinorNew"];
			vValue["nVersionPatch"] = vValue["nVersionPatchNew"];
			vValue["nUpdateable"] = 0;
			fpWriteLog("success","REM File: " .. kKey .. " succesfully prepared for download. Downloading now...");
		end
	end
	if sURLStrings then
		Interface.clipboardSetText(sURLStrings);
		if sType == "Rulesets" then
			Interface.openWindow("url","C:/DOEREMUHelpers/helper08.bat");
		elseif sType == "Extensions" then
			Interface.openWindow("url","C:/DOEREMUHelpers/helper09.bat");
		elseif sType == "Modules" then
			Interface.openWindow("url","C:/DOEREMUHelpers/helper10.bat");
		end
	end
	return;
end

function fpUpdateManifest(sManifestFileName)
	local oNode = DB.findNode("MJB_REMU");
	local oRNode = oNode.getChild("rulesets");
	if oRNode then
		fpUpdateManifestDetails("aRulesets",oRNode,sManifestFileName);
	end
	local oENode = oNode.getChild("extensions");
	if oENode then
		fpUpdateManifestDetails("aExtensions",oENode,sManifestFileName);
	end
	local oMNode = oNode.getChild("modules");
	if oMNode then
		fpUpdateManifestDetails("aModules",oMNode,sManifestFileName);
	end
	return
end

function fpUpdateManifestDetails(sType,oNode,sManifestFileName)
	local aREM = GlobalRegistry.MJB_REMU[sType];
	for kKey,vValue in pairs(oNode.getChildren()) do
		local sFileName = vValue.getChild("sFileName").getValue();
		local nVerMaj = vValue.getChild("nVersionMajor").getValue();
		local nVerMin = vValue.getChild("nVersionMinor").getValue();
		local nVerPatch = vValue.getChild("nVersionPatch").getValue();
		if vValue.getChild("nDelete").getValue() == 1 then
			if aREM[sFileName] then
				aREM[sFileName] = nil;
				fpWriteLog("success","REM: " .. sFileName .. " removed.");
			end
		else
			if not aREM[sFileName] then
				aREM[sFileName] = {};
				aREM[sFileName]["sManifestName"] = sManifestFileName;
				aREM[sFileName]["nVersionMajor"] = 0;
				aREM[sFileName]["nVersionMinor"] = 0;
				aREM[sFileName]["nVersionPatch"] = 0;
				aREM[sFileName]["nIgnore"] = 0;
				aREM[sFileName]["nUpdateable"] = 0;
			end
			aREM[sFileName]["sURL"] = vValue.getChild("sURL").getValue();
			aREM[sFileName]["sProtocol"] = vValue.getChild("sProtocol").getValue();
			aREM[sFileName]["nVersionMajorNew"] = nVerMaj;
			aREM[sFileName]["nVersionMinorNew"] = nVerMin;
			aREM[sFileName]["nVersionPatchNew"] = nVerPatch;
			if aREM[sFileName]["nVersionMajor"] < nVerMaj or
					aREM[sFileName]["nVersionMajor"] == nVerMaj and
					aREM[sFileName]["nVersionMinor"] < nVerMin or
					aREM[sFileName]["nVersionMajor"] == nVerMaj and
					aREM[sFileName]["nVersionMinor"] == nVerMin and
					aREM[sFileName]["nVersionPatch"] < nVerPatch then
				aREM[sFileName]["nUpdateable"] = 1;
				fpWriteLog("success","REM: " .. sFileName .. " updateable.");
			else
				aREM[sFileName]["nUpdateable"] = 0;
				fpWriteLog("success","REM: " .. sFileName .. " checked for updates. No update indicated.");
			end
		end
	end
	fpSetMenubarButton();
	GlobalRegistry.MJB_REMU[sType] = aREM;
	local oWindow = Interface.findWindow("wcREMUWindow","");
	if oWindow then
		oWindow.swREMULists.subwindow.swRulesets.subwindow.wlList.fpCheckREMEntryExists("Rulesets");
		oWindow.swREMULists.subwindow.swExtensions.subwindow.wlList.fpCheckREMEntryExists("Extensions");
		oWindow.swREMULists.subwindow.swModules.subwindow.wlList.fpCheckREMEntryExists("Modules");
	end
	return;
end

function fpWriteLog(sLogType,sMsg)
	local oWindow;
	if sLogType == "success" then
		sLog = sLog .. "<p>" .. sMsg .. "</p>";
		oWindow = Interface.findWindow("wcLogWindow","");
		if oWindow then
			oWindow.sLogDisplay.setValue(sLog);
		end
	else
		sErrorLog = sErrorLog .. "<p>" .. sMsg .. "</p>";
		oWindow = Interface.findWindow("wcErrorLogWindow","");
		if oWindow then
			oWindow.sLogDisplay.setValue(sErrorLog);
		end
	end
	return;
end
